from mlflow.store.tracking.rest_store import RestStore
from mlflow.utils.rest_utils import MlflowHostCreds
from infinstor_mlflow_plugin.tokenfile import get_token
from os.path import expanduser
from os.path import sep as separator
from mlflow.entities import (
        ViewType
        )
import requests
from requests.exceptions import HTTPError
import json
from mlflow.entities.model_registry.model_version_stages import (
    get_canonical_stage,
    DEFAULT_STAGES_FOR_GET_LATEST_VERSIONS,
    STAGE_DELETED_INTERNAL,
    STAGE_ARCHIVED,
)

class CognitoModelVersion():
    def __init__(self, name, user_id, version, creation_timestamp, last_updated_timestamp,
            current_stage, source, run_id, status):
        self.name = name
        self.user_id = user_id
        self.version = version
        self.creation_timestamp = creation_timestamp
        self.last_updated_timestamp = last_updated_timestamp
        self.current_stage = current_stage
        self.source = source
        self.run_id = status

class CognitoAuthenticatedRestStore(RestStore):
    def cognito_host_creds(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + 'token'
        token, service = get_token(tokfile, False)
        return MlflowHostCreds('https://mlflow.' + service + ':443/', token=token)

    def get_service(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + 'token'
        token, service = get_token(tokfile, False)
        return service

    def get_token_string(self):
        tokfile = expanduser("~") + separator + '.infinstor' + separator + 'token'
        token, service = get_token(tokfile, False)
        return token

    def _hard_delete_run(self, run_id):
        """
        Permanently delete a run (metadata and metrics, tags, parameters).
        This is used by the ``mlflow gc`` command line and is not intended to be used elsewhere.
        """
        print('_hard_delete_run: Entered. run_id=' + str(run_id))
        headers = {
                'Content-Type': 'application/x-amz-json-1.1',
                'Authorization' : 'Bearer ' + self.get_token_string()
                }
        url = 'https://mlflow.' + self.get_service() + '/api/2.0/mlflow/runs/hard-delete'

        body = dict()
        body['run_id'] = run_id

        try:
            response = requests.post(url, data=json.dumps(body), headers=headers)
            response.raise_for_status()
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            raise
        except Exception as err:
            print(f'Other error occurred: {err}')
            raise
        else:
            return

    def _get_deleted_runs(self):
        print('_get_deleted_runs: Entered')
        experiments = self.list_experiments(view_type=ViewType.ALL)
        experiment_ids = map(lambda x: x.experiment_id, experiments)
        print('_get_deleted_runs: experiment_ids=' + str(experiment_ids))
        deleted_runs = self.search_runs(
            experiment_ids=experiment_ids, filter_string="", run_view_type=ViewType.DELETED_ONLY
        )
        rv = [deleted_run.info.run_uuid for deleted_run in deleted_runs]
        print('_get_deleted_runs: deleted_runs=' + str(rv))
        return rv

    def get_latest_versions(self, name, stages=None):
        """
        Latest version models for each requested stage. If no ``stages`` argument is provided,
        returns the latest version for each stage.

        :param name: Registered model name.
        :param stages: List of desired stages. If input list is None, return latest versions for
                       for 'Staging' and 'Production' stages.
        :return: List of :py:class:`mlflow.entities.model_registry.ModelVersion` objects.
        """
        #print('get_latest_versions: Entered. name=' + str(name) + ', stages=' + str(stages),
        #        flush=True)
        headers = {
                'Content-Type': 'application/x-amz-json-1.1',
                'Authorization' : 'Bearer ' + self.get_token_string()
                }
        url = 'https://mlflow.' + self.get_service() + '/api/2.0/mlflow/registered-models/get'

        try:
            response = requests.get(url, params={'name':name}, headers=headers)
            response.raise_for_status()
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            raise
        except Exception as err:
            print(f'Other error occurred: {err}')
            raise
        respjson = json.loads(response.text)
        model = respjson['registered_model']
        staging = None
        production = None
        archived = None
        for lv in model['latest_versions']:
            if (lv['current_stage'] == 'Staging'):
                if (not staging):
                    staging = lv
                elif (int(lv['version']) > int(staging['version'])):
                    staging = lv
            elif (lv['current_stage'] == 'Production'):
                if (not production):
                    production = lv
                elif (int(lv['version']) > int(production['version'])):
                    production = lv
            elif (lv['current_stage'] == 'Archived'):
                if (not archived):
                    archived = lv
                elif (int(lv['version']) > int(archived['version'])):
                    archived = lv

        latest_versions = []
        if (staging):
            latest_versions.append(CognitoModelVersion(staging['name'],
                staging['user_id'], staging['version'], staging['creation_timestamp'],
                staging['last_updated_timestamp'], staging['current_stage'],
                staging['source'], staging['run_id'], staging['status']))
        if (production):
            latest_versions.append(CognitoModelVersion(production['name'],
                production['user_id'], production['version'], production['creation_timestamp'],
                production['last_updated_timestamp'], production['current_stage'],
                production['source'], production['run_id'], production['status']))
        if (archived):
            latest_versions.append(CognitoModelVersion(archived['name'],
                archived['user_id'], archived['version'], archived['creation_timestamp'],
                archived['last_updated_timestamp'], archived['current_stage'],
                archived['source'], archived['run_id'], archived['status']))

        if stages is None or len(stages) == 0 or stages[0] == '':
            expected_stages = set(
                [get_canonical_stage(stage) for stage in DEFAULT_STAGES_FOR_GET_LATEST_VERSIONS]
            )
        else:
            expected_stages = set([get_canonical_stage(stage) for stage in stages])
        return [mv for mv in latest_versions if mv.current_stage in expected_stages]

    def get_model_version_download_uri(self, name, version):
        #print('get_model_version_download_uri: Entered. name=' + str(name)
        #        + ', version=' + str(version), flush=True)
        headers = {
                'Content-Type': 'application/x-amz-json-1.1',
                'Authorization' : 'Bearer ' + self.get_token_string()
                }
        url = 'https://mlflow.' + self.get_service() + '/api/2.0/mlflow/model-versions/get'

        try:
            response = requests.get(url, params={'name':name, 'version':version}, headers=headers)
            response.raise_for_status()
        except HTTPError as http_err:
            print(f'HTTP error occurred: {http_err}')
            raise
        except Exception as err:
            print(f'Other error occurred: {err}')
            raise
        respjson = json.loads(response.text)
        model_version = respjson['model_version']
        return model_version['source']

    def __init__(self, store_uri=None, artifact_uri=None):
        super().__init__(self.cognito_host_creds)




"""
The ``mlflow.sklearn`` module provides an API for logging and loading scikit-learn models. This
module exports scikit-learn models with the following flavors:

Python (native) `pickle <https://scikit-learn.org/stable/modules/model_persistence.html>`_ format
    This is the main flavor that can be loaded back into scikit-learn.

:py:mod:`mlflow.pyfunc`
    Produced for use by generic pyfunc-based deployment tools and batch inference.
"""

from __future__ import absolute_import

import math
import os
from multiprocessing import Pool
import torch
import yaml
import transformers
from transformers import BertConfig, BertForSequenceClassification, BertTokenizer, XLNetConfig, \
    XLNetForSequenceClassification, XLNetTokenizer, XLMConfig, XLMForSequenceClassification, XLMTokenizer, \
    RobertaConfig, RobertaForSequenceClassification, RobertaTokenizer, DistilBertConfig, \
    DistilBertForSequenceClassification, DistilBertTokenizer, InputExample, InputFeatures, \
    get_linear_schedule_with_warmup, AdamW
from transformers import glue_compute_metrics as compute_metrics
from transformers import glue_processors as processors
from transformers import glue_output_modes as output_modes
from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
from torch.utils.data.distributed import DistributedSampler
import numpy as np

import mlflow
from mlflow import pyfunc
from mlflow.exceptions import MlflowException
from mlflow.models import Model
from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
from mlflow.tracking.artifact_utils import _download_artifact_from_uri
from mlflow.utils.environment import _mlflow_conda_env
from mlflow.utils.model_utils import _get_flavor_configuration

from mlflow.utils.file_utils import TempDir
from infinstor_mlflow_plugin import transformers as infinstor_transformers

FLAVOR_NAME = "transformers"

MODEL_CLASSES = {
    'bert': (BertConfig, BertForSequenceClassification, BertTokenizer, 'bert-base-uncased'),
    'xlnet': (XLNetConfig, XLNetForSequenceClassification, XLNetTokenizer, 'xlnet-base-cased'),
    'xlm': (XLMConfig, XLMForSequenceClassification, XLMTokenizer, 'xlm-mlm-enfr-1024'),
    'roberta': (RobertaConfig, RobertaForSequenceClassification, RobertaTokenizer, 'distilbert-base-uncased'),
    'distilbert': (DistilBertConfig, DistilBertForSequenceClassification, DistilBertTokenizer, 'roberta-base'),
}

SERIALIZATION_FORMAT_PICKLE = "pickle"
SERIALIZATION_FORMAT_CLOUDPICKLE = "cloudpickle"

SUPPORTED_SERIALIZATION_FORMATS = [
    SERIALIZATION_FORMAT_PICKLE,
    SERIALIZATION_FORMAT_CLOUDPICKLE
]


def get_default_conda_env(include_cloudpickle=False):
    """
    :return: The default Conda environment for MLflow Models produced by calls to
             :func:`save_model()` and :func:`log_model()`.
    """
    pip_deps = None
    if include_cloudpickle:
        import cloudpickle
        pip_deps = ["cloudpickle=={}".format(cloudpickle.__version__)]
    return _mlflow_conda_env(
        additional_conda_deps=[
            "transformers={}".format(transformers.__version__),
        ],
        additional_pip_deps=pip_deps,
        additional_conda_channels=None
    )


def save_model(transformers_model, tokenizer, path, conda_env=None, mlflow_model=Model()):
    """
    Save a scikit-learn model to a path on the local file system.

    :param transformers_model: scikit-learn model to be saved.
    :param path: Local path where the model is to be saved.
    :param conda_env: Either a dictionary representation of a Conda environment or the path to a
                      Conda environment yaml file. If provided, this decsribes the environment
                      this model should be run in. At minimum, it should specify the dependencies
                      contained in :func:`get_default_conda_env()`. If `None`, the default
                      :func:`get_default_conda_env()` environment is added to the model.
                      The following is an *example* dictionary representation of a Conda
                      environment::

                        {
                            'name': 'mlflow-env',
                            'channels': ['defaults'],
                            'dependencies': [
                                'python=3.7.0',
                                'scikit-learn=0.19.2'
                            ]
                        }

    :param mlflow_model: :py:mod:`mlflow.models.Model` this flavor is being added to.
    :param tokenizer: tokenizer
    """
    if os.path.exists(path):
        raise MlflowException(message="Path '{}' already exists".format(path),
                              error_code=RESOURCE_ALREADY_EXISTS)
    os.makedirs(path)
    model_data_subpath = ""
    _save_model(transformers_model, tokenizer, model_path=os.path.join(path, model_data_subpath))
    conda_env_subpath = "conda.yaml"
    if conda_env is None:
        conda_env = get_default_conda_env(
            include_cloudpickle=SERIALIZATION_FORMAT_CLOUDPICKLE == SERIALIZATION_FORMAT_CLOUDPICKLE)
    elif not isinstance(conda_env, dict):
        with open(conda_env, "r") as f:
            conda_env = yaml.safe_load(f)
    with open(os.path.join(path, conda_env_subpath), "w") as f:
        yaml.safe_dump(conda_env, stream=f, default_flow_style=False)
    pyfunc.add_to_model(mlflow_model, loader_module="infinstor_mlflow_plugin.transformers", data=model_data_subpath,
                        env=conda_env_subpath)
    mlflow_model.add_flavor(FLAVOR_NAME,
                            pickled_model=model_data_subpath,
                            transformers_version=transformers.__version__)
    mlflow_model.save(os.path.join(path, "MLmodel"))


def log_model(model, tokenizer, conda_env=None):
    return transformers_log(artifact_path='infinstor/model',
                                  transformers_model=model,
                                  tokenizer=tokenizer,
                                  flavor=infinstor_transformers,
                                  registered_model_name=None,
                                  conda_env=conda_env)

def transformers_log(artifact_path, transformers_model, tokenizer, flavor, registered_model_name=None,
                     **kwargs):
    """
    Log model using supplied flavor module. If no run is active, this method will create a new
    active run.

    :param artifact_path: Run relative path identifying the model.
    :param transformers_model: The transformers model to log.
    :param tokenizer: The tokenizer to log.
    :param flavor: Flavor module to save the model with. The module must have
                   the ``save_model`` function that will persist the model as a valid
                   MLflow model.
    :param registered_model_name: Note:: Experimental: This argument may change or be removed
                                  in a future release without warning. If given, create a model
                                  version under ``registered_model_name``, also creating a
                                  registered model if one with the given name does not exist.
    :param kwargs: Extra args passed to the model flavor.
    """
    with TempDir() as tmp:
        local_path = tmp.path("model")
        save_model(transformers_model, tokenizer, local_path, None, Model())
        mlflow.tracking.fluent.log_artifacts(local_path, artifact_path)
        if registered_model_name is not None:
            run_id = mlflow.tracking.fluent.active_run().info.run_id
            mlflow.register_model("runs:/%s/%s" % (run_id, artifact_path),
                                  registered_model_name)

def _load_model_from_local_file(model_type, model_path):
    """Load a scikit-learn model saved as an MLflow artifact on the local file system."""
    config_class, model_class, tokenizer_class, _ = MODEL_CLASSES[model_type]
    config = config_class.from_pretrained(model_path)
    tokenizer = tokenizer_class.from_pretrained(model_path)
    model = model_class.from_pretrained(model_path, config=config)
    return model, tokenizer, config


def _load_pyfunc(path):
    """
    Load PyFunc implementation. Called by ``pyfunc.load_pyfunc``.

    :param path: Local filesystem path to the MLflow Model with the ``transformers`` flavor.
    """
    return _transformer_load_pyfunc(path)


def _transformer_load_pyfunc(path, model_type=None):
    if not model_type:
        model_type = "bert"
    model, tokenizer, config = _load_model_from_local_file(model_type, path)
    return _TransformersModelWrapper(model, tokenizer, config)

class _TransformersModelWrapper:
    def __init__(self, model, tokenizer, config):
        self.l_model = model
        self.l_tokenizer = tokenizer
        self.l_config = config

    def predict(self, dataframe):
        column_name = dataframe.columns[0]
        sentiments = []
        for headline in dataframe[column_name].values:
            max_len = 512
            n = 0
            sentiment = 0
            import nltk
            nltk.download('punkt')
            for input_sent in nltk.sent_tokenize(headline):
                n += 1
                import pandas as pd
                i_df = pd.DataFrame([input_sent])
                i_tokenized = i_df[0].apply((lambda x:self.l_tokenizer.encode(x, add_special_tokens=True)))
                i_padded = np.array([i + [0]*(max_len-len(i)) for i in i_tokenized.values])
                i_attention_mask = np.where(i_padded != 0, 1, 0)
                i_input_ids = torch.tensor(i_padded)  
                i_attention_mask = torch.tensor(i_attention_mask)
                i_predictions = []
                with torch.no_grad():
                    i_last_hidden_states = self.l_model(i_input_ids, attention_mask=i_attention_mask)

                i_logits = i_last_hidden_states[0]

                # Store predictions and true labels
                i_predictions.append(i_logits)
                i_pred_label = np.argmax(i_predictions[0], axis=1).flatten()

                #print('Sentiment for ' + input_sent + ' = ' + str(i_pred_label[0].numpy()))
                sentiment += i_pred_label[0].numpy()
            sentiments.append(int(round(sentiment/n)))

        rv = pd.Series(sentiments)
        return rv

def _save_model(transformers_model, tokenizer, model_path):
    """
    :param transformers_model: The scikit-learn model to serialize.
    :param model_path: The file path to which to write the serialized model.
    :param tokenizer: Tokenizer to save.
    """
    transformers_model.save_pretrained(model_path)
    tokenizer.save_pretrained(model_path)


def load_model(model_type, model_uri=None):
    """
    Load a transformer-learn model from a local file or a run.
    :param model_type: The type of transformer model we are using
    :param model_uri: The location, in URI format, of the MLflow model, for example:

                      - ``/Users/me/path/to/local/model``
                      - ``relative/path/to/local/model``
                      - ``s3://my_bucket/path/to/model``
                      - ``runs:/<mlflow_run_id>/run-relative/path/to/model``
                      - ``models:/<model_name>/<model_version>``
                      - ``models:/<model_name>/<stage>``

                      For more information about supported URI schemes, see
                      `Referencing Artifacts <https://www.mlflow.org/docs/latest/concepts.html#
                      artifact-locations>`_.

    :return: A transformers-learn model.
    """
    run_id = mlflow.tracking.fluent._get_or_start_run().info.run_id
    model_save_dir = os.path.join("runs:/", run_id)
    model_save_dir = os.path.join(model_save_dir, "model")
    if not model_uri:
        local_model_path = _download_artifact_from_uri(artifact_uri=model_save_dir)
    else:
        local_model_path = _download_artifact_from_uri(artifact_uri=model_uri)
    flavor_conf = _get_flavor_configuration(model_path=local_model_path, flavor_name=FLAVOR_NAME)
    transformers_model_artifacts_path = os.path.join(local_model_path, flavor_conf['pickled_model'])
    return _load_model_from_local_file(model_type=model_type, model_path=transformers_model_artifacts_path)



#!/usr/bin/env python
import sys
import getpass
import json
import builtins
from . import servicedefs
from infinstor_mlflow_plugin.tokenfile import read_token_file, write_token_file, get_token
from requests.exceptions import HTTPError
import requests
from os.path import expanduser
from os.path import sep as separator
import datetime
import configparser
from urllib.parse import unquote
import os

def get_creds():
    if sys.stdin.isatty():
        print("Enter your InfinStor service username and password")
        service = input("Service (e.g. infinstor.com): ")
        username = input("Username: ")
        password = getpass.getpass("Password: ")
    else:
        service = sys.stdin.readline().rstrip()
        username = sys.stdin.readline().rstrip()
        password = sys.stdin.readline().rstrip()
    return service, username, password

def login_and_update_token_file(service, username, password):
    url = 'https://service.' + service + '/assets/serviceconfig.js'
    try:
        response = requests.get(url)
        response.raise_for_status()
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
        raise
    except Exception as err:
        print(f'Other error occurred: {err}')
        raise
    s1 = response.text[response.text.index('CliClientId: "') + len('CliClientId: "'):]
    s2 = s1[:s1.index('"')]
    builtins.clientid = s2
    print('clientid=' + s2)

    postdata = dict()
    auth_parameters = dict()
    auth_parameters['USERNAME'] = username
    auth_parameters['PASSWORD'] = password
    postdata['AuthParameters'] = auth_parameters
    postdata['AuthFlow'] = "USER_PASSWORD_AUTH"
    postdata['ClientId'] = builtins.clientid

    payload = json.dumps(postdata)

    url = 'https://cognito-idp.us-east-1.amazonaws.com:443/'
    headers = {
            'Content-Type': 'application/x-amz-json-1.1',
            'X-Amz-Target' : 'AWSCognitoIdentityProviderService.InitiateAuth'
            }

    try:
        response = requests.post(url, data=payload, headers=headers)
        response.raise_for_status()
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
        raise
    except Exception as err:
        print(f'Other error occurred: {err}')
        raise


    authres = response.json()['AuthenticationResult']
    token = authres['IdToken']
    refresh_token = authres['RefreshToken']

    # Call cognito REST API getUser to get custom:serviceName
    url = 'https://cognito-idp.us-east-1.amazonaws.com:443/'
    body = dict()
    body['AccessToken'] = authres['AccessToken']
    body_s = json.dumps(body)
    headers = {
            'Content-Type': 'application/x-amz-json-1.1',
            'X-Amz-Target' : 'AWSCognitoIdentityProviderService.GetUser'
            }
    try:
        response = requests.post(url, data=body_s, headers=headers)
        response.raise_for_status()
    except HTTPError as http_err:
        print(f'HTTP error occurred in getUser: {http_err}')
        raise
    except Exception as err:
        print(f'Other error occurred in getUser: {err}')
        raise
    else:
        print('cognito getUser success')
        user = response.json()
        useratt = user['UserAttributes']
        for oneattr in useratt:
            if (oneattr['Name'] == 'custom:serviceName'):
                builtins.service = oneattr['Value']
                print('Found serviceName ' + builtins.service + ' in cognito user')
                break
    if (builtins.service == None):
        print('Could not determine service')
        raise Exception('login', 'Could not determine service')

    token_time = round(datetime.datetime.timestamp(datetime.datetime.utcnow()))
    tokfile = expanduser("~") + separator + '.infinstor' + separator + 'token'
    write_token_file(tokfile, token_time, token, refresh_token, builtins.clientid,\
                builtins.service)

    payload = ("ProductCode=" + builtins.prodcode)
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': token
        }

    url = 'https://api.' + builtins.service + '/customerinfo'
    try:
        response = requests.post(url, data=payload, headers=headers)
        response.raise_for_status()
    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')
        raise
    except Exception as err:
        print(f'Other error occurred: {err}')
        raise

    # print('customerinfo success')
    response_json = response.json()
    infinStorAccessKeyId = unquote(response_json.get('InfinStorAccessKeyId'))
    infinStorSecretAccessKey = unquote(response_json.get('InfinStorSecretAccessKey'))
    setup_credentials(infinStorAccessKeyId, infinStorSecretAccessKey)

    return True

def setup_credentials(infinStorAccessKeyId, infinStorSecretAccessKey):
    home = expanduser("~")
    config = configparser.ConfigParser()
    newconfig = configparser.ConfigParser()
    credsfile = home + separator + ".aws" + separator + "credentials"
    if (os.path.exists(credsfile)):
        credsfile_save = home + separator + ".aws" + separator + "credentials.save"
        try:
            os.remove(credsfile_save)
        except Exception as err:
            print()
        try:
            os.rename(credsfile, credsfile_save)
        except Exception as err:
            print()
        config.read(credsfile_save)
        for section in config.sections():
            if (section != 'infinstor'):
                newconfig[section] = {}
                dct = dict(config[section])
                for key in dct:
                    newconfig[section][key] = dct[key]
    else:
        dotaws = home + "/.aws"
        if (os.path.exists(dotaws) == False):
            os.mkdir(dotaws, 0o755)
            open(credsfile, 'a').close()

    newconfig['infinstor'] = {}
    newconfig['infinstor']['aws_access_key_id'] = infinStorAccessKeyId
    newconfig['infinstor']['aws_secret_access_key'] = infinStorSecretAccessKey

    with open(credsfile, 'w') as configfile:
        newconfig.write(configfile)

def main():
    try:
        tokfile = expanduser("~") + separator + '.infinstor' + separator + 'token'
        token, service = get_token(tokfile, False)
        print('Login to service ' + service + ' complete')
        return True
    except Exception as err:
        pass

    service, username, password = get_creds()
    return login_and_update_token_file(service, username, password)

if __name__ == "__main__":
    if (main()):
        exit(0)
    else:
        exit(255)



from mlflow.store.artifact.s3_artifact_repo import S3ArtifactRepository


class InfinStorArtifactRepository(S3ArtifactRepository):
    """LocalArtifactRepository provided through plugin system"""
    is_plugin = True



from setuptools import setup, find_packages


setup(
    name="infinstor-mlflow-plugin",
    version="0.4.4",
    description="InfinStor plugin for MLflow",
    packages=find_packages(),
    # Require MLflow as a dependency of the plugin, so that plugin users can simply install
    # the plugin & then immediately use it with MLflow
    install_requires=["mlflow"],
    entry_points={
        # Define a Tracking Store plugin for tracking URIs with scheme 'infinstor'
        "mlflow.tracking_store": "infinstor=infinstor_mlflow_plugin.cognito_auth_rest_store:CognitoAuthenticatedRestStore",
        # Define a ArtifactRepository plugin for artifact URIs with scheme 'infinstor'
        "mlflow.artifact_repository":
            "infinstor=infinstor_mlflow_plugin.infinstor_artifact:InfinStorArtifactRepository",
        # Define a RunContextProvider plugin. The entry point name for run context providers
        # is not used, and so is set to the string "unused" here
        "mlflow.run_context_provider":
            "unused=infinstor_mlflow_plugin.run_context_provider:PluginRunContextProvider",
        # Define a Model Registry Store plugin for tracking URIs with scheme 'infinstor'
        "mlflow.model_registry_store":
            "infinstor=infinstor_mlflow_plugin.cognito_auth_rest_store:CognitoAuthenticatedRestStore",
        # Define a MLflow Project Backend plugin called 'infinstor-backend'
        "mlflow.project_backend":
            "infinstor-backend=infinstor_mlflow_plugin.infinstor_backend:PluginInfinStorProjectBackend",
        # Define a MLflow model deployment plugin for target 'faketarget'
        "mlflow.deployments": "infinstor=infinstor_mlflow_plugin.infinstor_deployment_plugin",
    },
)
